@@ -8,7 +8,7 @@ from retry import retry
 from .download_bitwarden import DownloadBitwarden
 
 
-class BitwardenTimeout(Exception):
+class BitwardenServerException(Exception):
     pass
 
 
@@ -27,8 +27,6 @@ class Bitwarden(object):
         self.bitwarden_credentials = bitwarden_credentials
         self.session_key = ""
 
-        self.bitwarden_exe("logout")
-
     def bitwarden_exe(self, *command):
         """
         Provide coma-separated command line arguments that you want to provide to bw CLI binary
@@ -82,12 +80,14 @@ class Bitwarden(object):
                 env=env,
             )
 
-    @retry(BitwardenTimeout, delay=5, tries=3)
+    @retry(BitwardenServerException, delay=5, tries=3)
     def bitwarden_login(self):
         """
         Performs login opeartion via BitWarden CLI
         Requires password / client_id / client_secret already set when creation Bitwarden instance
         """
+        self.bitwarden_exe("logout")
+
         bitwarden_app = self.bitwarden_exe(
             "login",
             "--apikey",
@@ -115,13 +115,12 @@ class Bitwarden(object):
             logging.error("STDOUT: " + bitwarden_app.stdout)
             logging.error("STDERR: " + bitwarden_app.stderr)
 
-            if "524" in bitwarden_app.stderr:
-                raise BitwardenTimeout("Timeout during login! Retrying...")
-
             if "invalid_client" in bitwarden_app.stderr:
                 raise Exception("Invalid bitwarden client_id or client_secret!")
 
-            raise Exception("Unknown issue during login!")
+            raise BitwardenServerException(
+                "Bitwarden Server error during login! Retrying..."
+            )
 
     def get_credentials(self, user_credentials_name):
         """
@@ -141,14 +140,21 @@ class Bitwarden(object):
 
         Example:
 
-          creds = {
+          items = {
               "unicourt_api": "UniCourt API",
               "unicourt_alpha_api": "UniCourt Alpha API Dev Portal",
               "aws": "AWS Access Key & S3 Bucket",
           }
-          bw.get_data(creds)
+          bw.get_data(items)
           assert isinstance(bw.data['aws'],dict)
         """
+        print("Sync bitwarden data...")
+        bitwarden_app = self.bitwarden_exe(
+            "sync",
+            "--session",
+            self.session_key,
+        )
+
         print("Getting bitwarden data...")
         bitwarden_app = self.bitwarden_exe(
             "list",
@@ -193,4 +199,74 @@ class Bitwarden(object):
             raise Exception(
                 "Invalid bitwarden collection or key name or no access to collection for this user!"
             )
-        self.bitwarden_exe("logout")
+
+    def get_attachment(self, item_name: str, file_name: str, output_folder: str = os.getcwd()):
+        """
+        Downloads attachment file from particular item to current working directory
+        Item name should be unique
+        File name should be unique
+
+        Example:
+
+            items = {
+                "test": "pypi.org",
+            }
+            self.bw.bitwarden_login()
+            self.bw.get_attachment(items["test"], "att.txt")
+            f = open("att.txt", "r")
+            assert f.read() == "secret text\n"
+        """
+        if not isinstance(item_name, str) or not isinstance(file_name, str):
+            raise Exception("item_name and file_name should be strings!")
+        if not os.path.exists(output_folder):
+            os.mkdir(output_folder)
+
+        # Get item ID
+        bitwarden_app = self.bitwarden_exe(
+            "get",
+            "item",
+            item_name,
+            "--session",
+            self.session_key,
+        )
+
+        if bitwarden_app.stderr:
+            logging.error("STDOUT: " + bitwarden_app.stdout)
+            logging.error("STDERR: " + bitwarden_app.stderr)
+            if "More than one result was found" in bitwarden_app.stderr:
+                raise Exception(
+                    "More than one result for item name was found! Name should be unique!"
+                )
+            if "Not found" in bitwarden_app.stderr:
+                raise Exception(
+                    "Cannot find bitwarden item '{}'! Check the name".format(
+                        str(item_name)
+                    )
+                )
+            raise Exception("Unknown error during items obtaining!")
+
+        bitwarden_items = json.loads(bitwarden_app.stdout)
+        item_id = str(bitwarden_items["id"])
+
+        bitwarden_app = self.bitwarden_exe(
+            "get",
+            "attachment",
+            file_name,
+            "--itemid",
+            item_id,
+            "--session",
+            self.session_key,
+            "--output",
+            os.path.join(output_folder, file_name)
+        )
+
+        if bitwarden_app.stderr:
+            logging.error("STDOUT: " + bitwarden_app.stdout)
+            logging.error("STDERR: " + bitwarden_app.stderr)
+
+            if "was not found" in bitwarden_app.stderr:
+                raise Exception("Attachment was not found!")
+
+            raise Exception("Unknown error during attachment downloading!")
+
+        print("Attachment '{}' downloaded sucessfully!".format(file_name))
